@@ -3,6 +3,7 @@
 #
 # ISO 15.2.18
 class Struct
+  include Enumerable
 
   ##
   # Calls the given block for each element of +self+
@@ -44,35 +45,6 @@ class Struct
     ary
   end
 
-  def _inspect(recur_list)
-    return "#<struct #{self.class}:...>" if recur_list[self.object_id]
-    recur_list[self.object_id] = true
-    name = self.class.to_s
-    if name[0] == "#"
-      str = "#<struct "
-    else
-      str = "#<struct #{name} "
-    end
-    buf = []
-    self.each_pair do |k,v|
-      buf.push k.to_s + "=" + v._inspect(recur_list)
-    end
-    str + buf.join(", ") + ">"
-  end
-
-  ##
-  # call-seq:
-  #   struct.to_s      -> string
-  #   struct.inspect   -> string
-  #
-  # Describe the contents of this struct in a string.
-  #
-  # 15.2.18.4.10(x)
-  #
-  def inspect
-    self._inspect({})
-  end
-
   ##
   # 15.2.18.4.11(x)
   #
