@@ -49,12 +49,6 @@ mrb_p(mrb_state *mrb, mrb_value obj)
 }
 
 
-MRB_API void
-mrb_print_error(mrb_state *mrb)
-{
-  mrb_print_backtrace(mrb);
-}
-
 MRB_API void
 mrb_show_version(mrb_state *mrb)
 {
@@ -78,11 +72,6 @@ mrb_p(mrb_state *mrb, mrb_value obj)
 {
 }
 
-MRB_API void
-mrb_print_error(mrb_state *mrb)
-{
-}
-
 MRB_API void
 mrb_show_version(mrb_state *mrb)
 {
